
MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:basic",
    "fro": {
        'urn:mace:dir:attribute-def:aRecord': 'aRecord',
        'urn:mace:dir:attribute-def:aliasedEntryName': 'aliasedEntryName',
        'urn:mace:dir:attribute-def:aliasedObjectName': 'aliasedObjectName',
        'urn:mace:dir:attribute-def:associatedDomain': 'associatedDomain',
        'urn:mace:dir:attribute-def:associatedName': 'associatedName',
        'urn:mace:dir:attribute-def:audio': 'audio',
        'urn:mace:dir:attribute-def:authorityRevocationList': 'authorityRevocationList',
        'urn:mace:dir:attribute-def:buildingName': 'buildingName',
        'urn:mace:dir:attribute-def:businessCategory': 'businessCategory',
        'urn:mace:dir:attribute-def:c': 'c',
        'urn:mace:dir:attribute-def:cACertificate': 'cACertificate',
        'urn:mace:dir:attribute-def:cNAMERecord': 'cNAMERecord',
        'urn:mace:dir:attribute-def:carLicense': 'carLicense',
        'urn:mace:dir:attribute-def:certificateRevocationList': 'certificateRevocationList',
        'urn:mace:dir:attribute-def:cn': 'cn',
        'urn:mace:dir:attribute-def:co': 'co',
        'urn:mace:dir:attribute-def:commonName': 'commonName',
        'urn:mace:dir:attribute-def:countryName': 'countryName',
        'urn:mace:dir:attribute-def:crossCertificatePair': 'crossCertificatePair',
        'urn:mace:dir:attribute-def:dITRedirect': 'dITRedirect',
        'urn:mace:dir:attribute-def:dSAQuality': 'dSAQuality',
        'urn:mace:dir:attribute-def:dc': 'dc',
        'urn:mace:dir:attribute-def:deltaRevocationList': 'deltaRevocationList',
        'urn:mace:dir:attribute-def:departmentNumber': 'departmentNumber',
        'urn:mace:dir:attribute-def:description': 'description',
        'urn:mace:dir:attribute-def:destinationIndicator': 'destinationIndicator',
        'urn:mace:dir:attribute-def:displayName': 'displayName',
        'urn:mace:dir:attribute-def:distinguishedName': 'distinguishedName',
        'urn:mace:dir:attribute-def:dmdName': 'dmdName',
        'urn:mace:dir:attribute-def:dnQualifier': 'dnQualifier',
        'urn:mace:dir:attribute-def:documentAuthor': 'documentAuthor',
        'urn:mace:dir:attribute-def:documentIdentifier': 'documentIdentifier',
        'urn:mace:dir:attribute-def:documentLocation': 'documentLocation',
        'urn:mace:dir:attribute-def:documentPublisher': 'documentPublisher',
        'urn:mace:dir:attribute-def:documentTitle': 'documentTitle',
        'urn:mace:dir:attribute-def:documentVersion': 'documentVersion',
        'urn:mace:dir:attribute-def:domainComponent': 'domainComponent',
        'urn:mace:dir:attribute-def:drink': 'drink',
        'urn:mace:dir:attribute-def:eduOrgHomePageURI': 'eduOrgHomePageURI',
        'urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI': 'eduOrgIdentityAuthNPolicyURI',
        'urn:mace:dir:attribute-def:eduOrgLegalName': 'eduOrgLegalName',
        'urn:mace:dir:attribute-def:eduOrgSuperiorURI': 'eduOrgSuperiorURI',
        'urn:mace:dir:attribute-def:eduOrgWhitePagesURI': 'eduOrgWhitePagesURI',
        'urn:mace:dir:attribute-def:eduPersonAffiliation': 'eduPersonAffiliation',
        'urn:mace:dir:attribute-def:eduPersonEntitlement': 'eduPersonEntitlement',
        'urn:mace:dir:attribute-def:eduPersonNickname': 'eduPersonNickname',
        'urn:mace:dir:attribute-def:eduPersonOrgDN': 'eduPersonOrgDN',
        'urn:mace:dir:attribute-def:eduPersonOrgUnitDN': 'eduPersonOrgUnitDN',
        'urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation': 'eduPersonPrimaryAffiliation',
        'urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN': 'eduPersonPrimaryOrgUnitDN',
        'urn:mace:dir:attribute-def:eduPersonPrincipalName': 'eduPersonPrincipalName',
        'urn:mace:dir:attribute-def:eduPersonScopedAffiliation': 'eduPersonScopedAffiliation',
        'urn:mace:dir:attribute-def:eduPersonTargetedID': 'eduPersonTargetedID',
        'urn:mace:dir:attribute-def:email': 'email',
        'urn:mace:dir:attribute-def:emailAddress': 'emailAddress',
        'urn:mace:dir:attribute-def:employeeNumber': 'employeeNumber',
        'urn:mace:dir:attribute-def:employeeType': 'employeeType',
        'urn:mace:dir:attribute-def:enhancedSearchGuide': 'enhancedSearchGuide',
        'urn:mace:dir:attribute-def:facsimileTelephoneNumber': 'facsimileTelephoneNumber',
        'urn:mace:dir:attribute-def:favouriteDrink': 'favouriteDrink',
        'urn:mace:dir:attribute-def:fax': 'fax',
        'urn:mace:dir:attribute-def:federationFeideSchemaVersion': 'federationFeideSchemaVersion',
        'urn:mace:dir:attribute-def:friendlyCountryName': 'friendlyCountryName',
        'urn:mace:dir:attribute-def:generationQualifier': 'generationQualifier',
        'urn:mace:dir:attribute-def:givenName': 'givenName',
        'urn:mace:dir:attribute-def:gn': 'gn',
        'urn:mace:dir:attribute-def:homePhone': 'homePhone',
        'urn:mace:dir:attribute-def:homePostalAddress': 'homePostalAddress',
        'urn:mace:dir:attribute-def:homeTelephoneNumber': 'homeTelephoneNumber',
        'urn:mace:dir:attribute-def:host': 'host',
        'urn:mace:dir:attribute-def:houseIdentifier': 'houseIdentifier',
        'urn:mace:dir:attribute-def:info': 'info',
        'urn:mace:dir:attribute-def:initials': 'initials',
        'urn:mace:dir:attribute-def:internationaliSDNNumber': 'internationaliSDNNumber',
        'urn:mace:dir:attribute-def:janetMailbox': 'janetMailbox',
        'urn:mace:dir:attribute-def:jpegPhoto': 'jpegPhoto',
        'urn:mace:dir:attribute-def:knowledgeInformation': 'knowledgeInformation',
        'urn:mace:dir:attribute-def:l': 'l',
        'urn:mace:dir:attribute-def:labeledURI': 'labeledURI',
        'urn:mace:dir:attribute-def:localityName': 'localityName',
        'urn:mace:dir:attribute-def:mDRecord': 'mDRecord',
        'urn:mace:dir:attribute-def:mXRecord': 'mXRecord',
        'urn:mace:dir:attribute-def:mail': 'mail',
        'urn:mace:dir:attribute-def:mailPreferenceOption': 'mailPreferenceOption',
        'urn:mace:dir:attribute-def:manager': 'manager',
        'urn:mace:dir:attribute-def:member': 'member',
        'urn:mace:dir:attribute-def:mobile': 'mobile',
        'urn:mace:dir:attribute-def:mobileTelephoneNumber': 'mobileTelephoneNumber',
        'urn:mace:dir:attribute-def:nSRecord': 'nSRecord',
        'urn:mace:dir:attribute-def:name': 'name',
        'urn:mace:dir:attribute-def:norEduOrgAcronym': 'norEduOrgAcronym',
        'urn:mace:dir:attribute-def:norEduOrgNIN': 'norEduOrgNIN',
        'urn:mace:dir:attribute-def:norEduOrgSchemaVersion': 'norEduOrgSchemaVersion',
        'urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier': 'norEduOrgUniqueIdentifier',
        'urn:mace:dir:attribute-def:norEduOrgUniqueNumber': 'norEduOrgUniqueNumber',
        'urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier': 'norEduOrgUnitUniqueIdentifier',
        'urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber': 'norEduOrgUnitUniqueNumber',
        'urn:mace:dir:attribute-def:norEduPersonBirthDate': 'norEduPersonBirthDate',
        'urn:mace:dir:attribute-def:norEduPersonLIN': 'norEduPersonLIN',
        'urn:mace:dir:attribute-def:norEduPersonNIN': 'norEduPersonNIN',
        'urn:mace:dir:attribute-def:o': 'o',
        'urn:mace:dir:attribute-def:objectClass': 'objectClass',
        'urn:mace:dir:attribute-def:organizationName': 'organizationName',
        'urn:mace:dir:attribute-def:organizationalStatus': 'organizationalStatus',
        'urn:mace:dir:attribute-def:organizationalUnitName': 'organizationalUnitName',
        'urn:mace:dir:attribute-def:otherMailbox': 'otherMailbox',
        'urn:mace:dir:attribute-def:ou': 'ou',
        'urn:mace:dir:attribute-def:owner': 'owner',
        'urn:mace:dir:attribute-def:pager': 'pager',
        'urn:mace:dir:attribute-def:pagerTelephoneNumber': 'pagerTelephoneNumber',
        'urn:mace:dir:attribute-def:personalSignature': 'personalSignature',
        'urn:mace:dir:attribute-def:personalTitle': 'personalTitle',
        'urn:mace:dir:attribute-def:photo': 'photo',
        'urn:mace:dir:attribute-def:physicalDeliveryOfficeName': 'physicalDeliveryOfficeName',
        'urn:mace:dir:attribute-def:pkcs9email': 'pkcs9email',
        'urn:mace:dir:attribute-def:postOfficeBox': 'postOfficeBox',
        'urn:mace:dir:attribute-def:postalAddress': 'postalAddress',
        'urn:mace:dir:attribute-def:postalCode': 'postalCode',
        'urn:mace:dir:attribute-def:preferredDeliveryMethod': 'preferredDeliveryMethod',
        'urn:mace:dir:attribute-def:preferredLanguage': 'preferredLanguage',
        'urn:mace:dir:attribute-def:presentationAddress': 'presentationAddress',
        'urn:mace:dir:attribute-def:protocolInformation': 'protocolInformation',
        'urn:mace:dir:attribute-def:pseudonym': 'pseudonym',
        'urn:mace:dir:attribute-def:registeredAddress': 'registeredAddress',
        'urn:mace:dir:attribute-def:rfc822Mailbox': 'rfc822Mailbox',
        'urn:mace:dir:attribute-def:roleOccupant': 'roleOccupant',
        'urn:mace:dir:attribute-def:roomNumber': 'roomNumber',
        'urn:mace:dir:attribute-def:sOARecord': 'sOARecord',
        'urn:mace:dir:attribute-def:searchGuide': 'searchGuide',
        'urn:mace:dir:attribute-def:secretary': 'secretary',
        'urn:mace:dir:attribute-def:seeAlso': 'seeAlso',
        'urn:mace:dir:attribute-def:serialNumber': 'serialNumber',
        'urn:mace:dir:attribute-def:singleLevelQuality': 'singleLevelQuality',
        'urn:mace:dir:attribute-def:sn': 'sn',
        'urn:mace:dir:attribute-def:st': 'st',
        'urn:mace:dir:attribute-def:stateOrProvinceName': 'stateOrProvinceName',
        'urn:mace:dir:attribute-def:street': 'street',
        'urn:mace:dir:attribute-def:streetAddress': 'streetAddress',
        'urn:mace:dir:attribute-def:subtreeMaximumQuality': 'subtreeMaximumQuality',
        'urn:mace:dir:attribute-def:subtreeMinimumQuality': 'subtreeMinimumQuality',
        'urn:mace:dir:attribute-def:supportedAlgorithms': 'supportedAlgorithms',
        'urn:mace:dir:attribute-def:supportedApplicationContext': 'supportedApplicationContext',
        'urn:mace:dir:attribute-def:surname': 'surname',
        'urn:mace:dir:attribute-def:telephoneNumber': 'telephoneNumber',
        'urn:mace:dir:attribute-def:teletexTerminalIdentifier': 'teletexTerminalIdentifier',
        'urn:mace:dir:attribute-def:telexNumber': 'telexNumber',
        'urn:mace:dir:attribute-def:textEncodedORAddress': 'textEncodedORAddress',
        'urn:mace:dir:attribute-def:title': 'title',
        'urn:mace:dir:attribute-def:uid': 'uid',
        'urn:mace:dir:attribute-def:uniqueIdentifier': 'uniqueIdentifier',
        'urn:mace:dir:attribute-def:uniqueMember': 'uniqueMember',
        'urn:mace:dir:attribute-def:userCertificate': 'userCertificate',
        'urn:mace:dir:attribute-def:userClass': 'userClass',
        'urn:mace:dir:attribute-def:userPKCS12': 'userPKCS12',
        'urn:mace:dir:attribute-def:userPassword': 'userPassword',
        'urn:mace:dir:attribute-def:userSMIMECertificate': 'userSMIMECertificate',
        'urn:mace:dir:attribute-def:userid': 'userid',
        'urn:mace:dir:attribute-def:x121Address': 'x121Address',
        'urn:mace:dir:attribute-def:x500UniqueIdentifier': 'x500UniqueIdentifier',
        },
    "to": {
        'aRecord': 'urn:mace:dir:attribute-def:aRecord',
        'aliasedEntryName': 'urn:mace:dir:attribute-def:aliasedEntryName',
        'aliasedObjectName': 'urn:mace:dir:attribute-def:aliasedObjectName',
        'associatedDomain': 'urn:mace:dir:attribute-def:associatedDomain',
        'associatedName': 'urn:mace:dir:attribute-def:associatedName',
        'audio': 'urn:mace:dir:attribute-def:audio',
        'authorityRevocationList': 'urn:mace:dir:attribute-def:authorityRevocationList',
        'buildingName': 'urn:mace:dir:attribute-def:buildingName',
        'businessCategory': 'urn:mace:dir:attribute-def:businessCategory',
        'c': 'urn:mace:dir:attribute-def:c',
        'cACertificate': 'urn:mace:dir:attribute-def:cACertificate',
        'cNAMERecord': 'urn:mace:dir:attribute-def:cNAMERecord',
        'carLicense': 'urn:mace:dir:attribute-def:carLicense',
        'certificateRevocationList': 'urn:mace:dir:attribute-def:certificateRevocationList',
        'cn': 'urn:mace:dir:attribute-def:cn',
        'co': 'urn:mace:dir:attribute-def:co',
        'commonName': 'urn:mace:dir:attribute-def:commonName',
        'countryName': 'urn:mace:dir:attribute-def:countryName',
        'crossCertificatePair': 'urn:mace:dir:attribute-def:crossCertificatePair',
        'dITRedirect': 'urn:mace:dir:attribute-def:dITRedirect',
        'dSAQuality': 'urn:mace:dir:attribute-def:dSAQuality',
        'dc': 'urn:mace:dir:attribute-def:dc',
        'deltaRevocationList': 'urn:mace:dir:attribute-def:deltaRevocationList',
        'departmentNumber': 'urn:mace:dir:attribute-def:departmentNumber',
        'description': 'urn:mace:dir:attribute-def:description',
        'destinationIndicator': 'urn:mace:dir:attribute-def:destinationIndicator',
        'displayName': 'urn:mace:dir:attribute-def:displayName',
        'distinguishedName': 'urn:mace:dir:attribute-def:distinguishedName',
        'dmdName': 'urn:mace:dir:attribute-def:dmdName',
        'dnQualifier': 'urn:mace:dir:attribute-def:dnQualifier',
        'documentAuthor': 'urn:mace:dir:attribute-def:documentAuthor',
        'documentIdentifier': 'urn:mace:dir:attribute-def:documentIdentifier',
        'documentLocation': 'urn:mace:dir:attribute-def:documentLocation',
        'documentPublisher': 'urn:mace:dir:attribute-def:documentPublisher',
        'documentTitle': 'urn:mace:dir:attribute-def:documentTitle',
        'documentVersion': 'urn:mace:dir:attribute-def:documentVersion',
        'domainComponent': 'urn:mace:dir:attribute-def:domainComponent',
        'drink': 'urn:mace:dir:attribute-def:drink',
        'eduOrgHomePageURI': 'urn:mace:dir:attribute-def:eduOrgHomePageURI',
        'eduOrgIdentityAuthNPolicyURI': 'urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI',
        'eduOrgLegalName': 'urn:mace:dir:attribute-def:eduOrgLegalName',
        'eduOrgSuperiorURI': 'urn:mace:dir:attribute-def:eduOrgSuperiorURI',
        'eduOrgWhitePagesURI': 'urn:mace:dir:attribute-def:eduOrgWhitePagesURI',
        'eduPersonAffiliation': 'urn:mace:dir:attribute-def:eduPersonAffiliation',
        'eduPersonEntitlement': 'urn:mace:dir:attribute-def:eduPersonEntitlement',
        'eduPersonNickname': 'urn:mace:dir:attribute-def:eduPersonNickname',
        'eduPersonOrgDN': 'urn:mace:dir:attribute-def:eduPersonOrgDN',
        'eduPersonOrgUnitDN': 'urn:mace:dir:attribute-def:eduPersonOrgUnitDN',
        'eduPersonPrimaryAffiliation': 'urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation',
        'eduPersonPrimaryOrgUnitDN': 'urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN',
        'eduPersonPrincipalName': 'urn:mace:dir:attribute-def:eduPersonPrincipalName',
        'eduPersonScopedAffiliation': 'urn:mace:dir:attribute-def:eduPersonScopedAffiliation',
        'eduPersonTargetedID': 'urn:mace:dir:attribute-def:eduPersonTargetedID',
        'email': 'urn:mace:dir:attribute-def:email',
        'emailAddress': 'urn:mace:dir:attribute-def:emailAddress',
        'employeeNumber': 'urn:mace:dir:attribute-def:employeeNumber',
        'employeeType': 'urn:mace:dir:attribute-def:employeeType',
        'enhancedSearchGuide': 'urn:mace:dir:attribute-def:enhancedSearchGuide',
        'facsimileTelephoneNumber': 'urn:mace:dir:attribute-def:facsimileTelephoneNumber',
        'favouriteDrink': 'urn:mace:dir:attribute-def:favouriteDrink',
        'fax': 'urn:mace:dir:attribute-def:fax',
        'federationFeideSchemaVersion': 'urn:mace:dir:attribute-def:federationFeideSchemaVersion',
        'friendlyCountryName': 'urn:mace:dir:attribute-def:friendlyCountryName',
        'generationQualifier': 'urn:mace:dir:attribute-def:generationQualifier',
        'givenName': 'urn:mace:dir:attribute-def:givenName',
        'gn': 'urn:mace:dir:attribute-def:gn',
        'homePhone': 'urn:mace:dir:attribute-def:homePhone',
        'homePostalAddress': 'urn:mace:dir:attribute-def:homePostalAddress',
        'homeTelephoneNumber': 'urn:mace:dir:attribute-def:homeTelephoneNumber',
        'host': 'urn:mace:dir:attribute-def:host',
        'houseIdentifier': 'urn:mace:dir:attribute-def:houseIdentifier',
        'info': 'urn:mace:dir:attribute-def:info',
        'initials': 'urn:mace:dir:attribute-def:initials',
        'internationaliSDNNumber': 'urn:mace:dir:attribute-def:internationaliSDNNumber',
        'janetMailbox': 'urn:mace:dir:attribute-def:janetMailbox',
        'jpegPhoto': 'urn:mace:dir:attribute-def:jpegPhoto',
        'knowledgeInformation': 'urn:mace:dir:attribute-def:knowledgeInformation',
        'l': 'urn:mace:dir:attribute-def:l',
        'labeledURI': 'urn:mace:dir:attribute-def:labeledURI',
        'localityName': 'urn:mace:dir:attribute-def:localityName',
        'mDRecord': 'urn:mace:dir:attribute-def:mDRecord',
        'mXRecord': 'urn:mace:dir:attribute-def:mXRecord',
        'mail': 'urn:mace:dir:attribute-def:mail',
        'mailPreferenceOption': 'urn:mace:dir:attribute-def:mailPreferenceOption',
        'manager': 'urn:mace:dir:attribute-def:manager',
        'member': 'urn:mace:dir:attribute-def:member',
        'mobile': 'urn:mace:dir:attribute-def:mobile',
        'mobileTelephoneNumber': 'urn:mace:dir:attribute-def:mobileTelephoneNumber',
        'nSRecord': 'urn:mace:dir:attribute-def:nSRecord',
        'name': 'urn:mace:dir:attribute-def:name',
        'norEduOrgAcronym': 'urn:mace:dir:attribute-def:norEduOrgAcronym',
        'norEduOrgNIN': 'urn:mace:dir:attribute-def:norEduOrgNIN',
        'norEduOrgSchemaVersion': 'urn:mace:dir:attribute-def:norEduOrgSchemaVersion',
        'norEduOrgUniqueIdentifier': 'urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier',
        'norEduOrgUniqueNumber': 'urn:mace:dir:attribute-def:norEduOrgUniqueNumber',
        'norEduOrgUnitUniqueIdentifier': 'urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier',
        'norEduOrgUnitUniqueNumber': 'urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber',
        'norEduPersonBirthDate': 'urn:mace:dir:attribute-def:norEduPersonBirthDate',
        'norEduPersonLIN': 'urn:mace:dir:attribute-def:norEduPersonLIN',
        'norEduPersonNIN': 'urn:mace:dir:attribute-def:norEduPersonNIN',
        'o': 'urn:mace:dir:attribute-def:o',
        'objectClass': 'urn:mace:dir:attribute-def:objectClass',
        'organizationName': 'urn:mace:dir:attribute-def:organizationName',
        'organizationalStatus': 'urn:mace:dir:attribute-def:organizationalStatus',
        'organizationalUnitName': 'urn:mace:dir:attribute-def:organizationalUnitName',
        'otherMailbox': 'urn:mace:dir:attribute-def:otherMailbox',
        'ou': 'urn:mace:dir:attribute-def:ou',
        'owner': 'urn:mace:dir:attribute-def:owner',
        'pager': 'urn:mace:dir:attribute-def:pager',
        'pagerTelephoneNumber': 'urn:mace:dir:attribute-def:pagerTelephoneNumber',
        'personalSignature': 'urn:mace:dir:attribute-def:personalSignature',
        'personalTitle': 'urn:mace:dir:attribute-def:personalTitle',
        'photo': 'urn:mace:dir:attribute-def:photo',
        'physicalDeliveryOfficeName': 'urn:mace:dir:attribute-def:physicalDeliveryOfficeName',
        'pkcs9email': 'urn:mace:dir:attribute-def:pkcs9email',
        'postOfficeBox': 'urn:mace:dir:attribute-def:postOfficeBox',
        'postalAddress': 'urn:mace:dir:attribute-def:postalAddress',
        'postalCode': 'urn:mace:dir:attribute-def:postalCode',
        'preferredDeliveryMethod': 'urn:mace:dir:attribute-def:preferredDeliveryMethod',
        'preferredLanguage': 'urn:mace:dir:attribute-def:preferredLanguage',
        'presentationAddress': 'urn:mace:dir:attribute-def:presentationAddress',
        'protocolInformation': 'urn:mace:dir:attribute-def:protocolInformation',
        'pseudonym': 'urn:mace:dir:attribute-def:pseudonym',
        'registeredAddress': 'urn:mace:dir:attribute-def:registeredAddress',
        'rfc822Mailbox': 'urn:mace:dir:attribute-def:rfc822Mailbox',
        'roleOccupant': 'urn:mace:dir:attribute-def:roleOccupant',
        'roomNumber': 'urn:mace:dir:attribute-def:roomNumber',
        'sOARecord': 'urn:mace:dir:attribute-def:sOARecord',
        'searchGuide': 'urn:mace:dir:attribute-def:searchGuide',
        'secretary': 'urn:mace:dir:attribute-def:secretary',
        'seeAlso': 'urn:mace:dir:attribute-def:seeAlso',
        'serialNumber': 'urn:mace:dir:attribute-def:serialNumber',
        'singleLevelQuality': 'urn:mace:dir:attribute-def:singleLevelQuality',
        'sn': 'urn:mace:dir:attribute-def:sn',
        'st': 'urn:mace:dir:attribute-def:st',
        'stateOrProvinceName': 'urn:mace:dir:attribute-def:stateOrProvinceName',
        'street': 'urn:mace:dir:attribute-def:street',
        'streetAddress': 'urn:mace:dir:attribute-def:streetAddress',
        'subtreeMaximumQuality': 'urn:mace:dir:attribute-def:subtreeMaximumQuality',
        'subtreeMinimumQuality': 'urn:mace:dir:attribute-def:subtreeMinimumQuality',
        'supportedAlgorithms': 'urn:mace:dir:attribute-def:supportedAlgorithms',
        'supportedApplicationContext': 'urn:mace:dir:attribute-def:supportedApplicationContext',
        'surname': 'urn:mace:dir:attribute-def:surname',
        'telephoneNumber': 'urn:mace:dir:attribute-def:telephoneNumber',
        'teletexTerminalIdentifier': 'urn:mace:dir:attribute-def:teletexTerminalIdentifier',
        'telexNumber': 'urn:mace:dir:attribute-def:telexNumber',
        'textEncodedORAddress': 'urn:mace:dir:attribute-def:textEncodedORAddress',
        'title': 'urn:mace:dir:attribute-def:title',
        'uid': 'urn:mace:dir:attribute-def:uid',
        'uniqueIdentifier': 'urn:mace:dir:attribute-def:uniqueIdentifier',
        'uniqueMember': 'urn:mace:dir:attribute-def:uniqueMember',
        'userCertificate': 'urn:mace:dir:attribute-def:userCertificate',
        'userClass': 'urn:mace:dir:attribute-def:userClass',
        'userPKCS12': 'urn:mace:dir:attribute-def:userPKCS12',
        'userPassword': 'urn:mace:dir:attribute-def:userPassword',
        'userSMIMECertificate': 'urn:mace:dir:attribute-def:userSMIMECertificate',
        'userid': 'urn:mace:dir:attribute-def:userid',
        'x121Address': 'urn:mace:dir:attribute-def:x121Address',
        'x500UniqueIdentifier': 'urn:mace:dir:attribute-def:x500UniqueIdentifier',
    }
}


#!/usr/bin/env python

try:
    from xml.etree import cElementTree as ElementTree
    if ElementTree.VERSION < '1.3.0':
        # cElementTree has no support for register_namespace
        # neither _namespace_map, thus we sacrify performance
        # for correctness
        from xml.etree import ElementTree
except ImportError:
    try:
        import cElementTree as ElementTree
    except ImportError:
        from elementtree import ElementTree
from defusedxml.common import EntitiesForbidden

from pytest import raises

import saml2.samlp as samlp
from saml2.samlp import NAMESPACE as SAMLP_NAMESPACE
from saml2 import soap

NAMESPACE = "http://schemas.xmlsoap.org/soap/envelope/"

example = """<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
    <Body>
        <samlp:Response xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" 
            xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" 
            ID="_6c3a4f8b9c2d" Version="2.0" IssueInstant="2004-03-27T08:42:00Z">
        <saml:Issuer>https://www.example.com/SAML</saml:Issuer>
        <Status>
        <StatusCode Value='urn:oasis:names:tc:SAML:2.0:status:Success'/>
        </Status>
        <saml:Assertion>
        <saml:Subject></saml:Subject>
        <saml:AttributeStatement></saml:AttributeStatement>
        </saml:Assertion>
        </samlp:Response>
    </Body>
</Envelope>
"""


def test_parse_soap_envelope():
    envelope = ElementTree.fromstring(example)
    assert envelope.tag == '{%s}Envelope' % NAMESPACE
    # How to check that it's the right type ?
    assert len(envelope) == 1
    body = envelope[0]
    assert body.tag == '{%s}Body' % NAMESPACE
    assert len(body) == 1
    saml_part = body[0]
    assert saml_part.tag == '{%s}Response' % SAMLP_NAMESPACE
    # {http://schemas.xmlsoap.org/soap/envelope/}Envelope


def test_make_soap_envelope():
    envelope = ElementTree.Element('')
    envelope.tag = '{%s}Envelope' % NAMESPACE
    body = ElementTree.Element('')
    body.tag = '{%s}Body' % NAMESPACE
    envelope.append(body)    
    request = samlp.AuthnRequest()
    request.become_child_element_of(body)

    assert envelope.tag == '{%s}Envelope' % NAMESPACE
    assert len(envelope) == 1
    body = envelope[0]
    assert body.tag == '{%s}Body' % NAMESPACE
    assert len(body) == 1
    saml_part = body[0]
    assert saml_part.tag == '{%s}AuthnRequest' % SAMLP_NAMESPACE


def test_parse_soap_enveloped_saml_thingy_xxe():
    xml = """<?xml version="1.0"?>
    <!DOCTYPE lolz [
    <!ENTITY lol "lol">
    <!ELEMENT lolz (#PCDATA)>
    <!ENTITY lol1 "&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;">
    ]>
    <lolz>&lol1;</lolz>
    """
    with raises(EntitiesForbidden):
        soap.parse_soap_enveloped_saml_thingy(xml, None)


def test_class_instances_from_soap_enveloped_saml_thingies_xxe():
    xml = """<?xml version="1.0"?>
    <!DOCTYPE lolz [
    <!ENTITY lol "lol">
    <!ELEMENT lolz (#PCDATA)>
    <!ENTITY lol1 "&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;">
    ]>
    <lolz>&lol1;</lolz>
    """
    with raises(soap.XmlParseError):
        soap.class_instances_from_soap_enveloped_saml_thingies(xml, None)


def test_open_soap_envelope_xxe():
    xml = """<?xml version="1.0"?>
    <!DOCTYPE lolz [
    <!ENTITY lol "lol">
    <!ELEMENT lolz (#PCDATA)>
    <!ENTITY lol1 "&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;">
    ]>
    <lolz>&lol1;</lolz>
    """
    with raises(soap.XmlParseError):
        soap.open_soap_envelope(xml)



"""This module provides methods for symmetric cryptography.

The default symmetric cryptography method used is Fernet by the cryptography
library. Reference: https://cryptography.io/en/latest/fernet/
"""

import base64 as _base64
import os as _os
import logging
from warnings import warn as _warn

import cryptography.fernet as _fernet
import cryptography.hazmat.backends as _backends
import cryptography.hazmat.primitives.ciphers as _ciphers

from .errors import SymmetricCryptographyError


logger = logging.getLogger(__name__)


class Fernet(object):
    """The default symmetric cryptography method."""

    @staticmethod
    def generate_key():
        """Return a key suitable for use by this method.

        :return: byte data representing the encyption/decryption key
        """
        key = _fernet.Fernet.generate_key()
        return key

    def __init__(self, key=None):
        """Initialize this method by optionally providing a key.

        :param key: byte data representing the encyption/decryption key
        """
        if key:
            fernet_key_error = SymmetricCryptographyError(
                "Fernet key must be 32 url-safe base64-encoded bytes."
            )
            try:
                raw_key = _base64.b64decode(key)
            except Exception as e:
                raise fernet_key_error from e
            else:
                if len(raw_key) != 32:
                    raise fernet_key_error
        else:
            key = self.__class__.generate_key()

        self._symmetric = _fernet.Fernet(key)

    def encrypt(self, plaintext, *args, **kwargs):
        """Encrypt the given plaintext.

        :param plaintext: byte data representing the plaintext
        :return: byte data representing the ciphertext
        """
        if args or kwargs:
            _deprecation_msg = (
                "The '.encrypt' method does not take into account any arguements, "
                "other than the 'ciphertext' param. "
                "Remove any other arguements. "
                "In the next version, this method will not allow them."
            )
            logger.warning(_deprecation_msg)
            _warn(_deprecation_msg, DeprecationWarning)

        ciphertext = self._symmetric.encrypt(plaintext)
        return ciphertext

    def decrypt(self, ciphertext, *args, **kwargs):
        """Decrypt the given ciphertext.

        :param ciphertext: byte data representing the ciphertext
        :return: byte data representing the plaintext
        """
        if args or kwargs:
            _deprecation_msg = (
                "The '.decrypt' method does not take into account any arguements, "
                "other than the 'ciphertext' param. "
                "Remove any other arguements. "
                "In the next version, this method will not allow them."
            )
            logger.warning(_deprecation_msg)
            _warn(_deprecation_msg, DeprecationWarning)

        plaintext = self._symmetric.decrypt(ciphertext)
        return plaintext

    def build_cipher(self, *args, **kwargs):
        _deprecation_msg = (
            "The 'Fernet' class does not need a build_cipher method."
            "Remove any calls to this method. "
            "In the next version, this method will be removed."
        )
        logger.warning(_deprecation_msg)
        _warn(_deprecation_msg, DeprecationWarning)


class AESCipher(object):
    """[deprecated] Symmetric cryptography method using AES.

    The default parameter set is AES 128bit in CBC mode.
    """

    POSTFIX_MODE = {
        'cbc': _ciphers.modes.CBC,
        'cfb': _ciphers.modes.CFB,
    }

    AES_BLOCK_SIZE = int(_ciphers.algorithms.AES.block_size / 8)

    @classmethod
    def _deprecation_notice(cls):
        """Warn about deprecation of this class."""
        _deprecation_msg = (
            '{name} {type} is deprecated. '
            'It will be removed in the next version. '
            'Use saml2.cryptography.symmetric.Default '
            'or saml2.cryptography.symmetric.Fernet '
            'instead.'
        ).format(name=cls.__name__, type=type(cls).__name__)
        logger.warning(_deprecation_msg)
        _warn(_deprecation_msg, DeprecationWarning)

    def __init__(self, key):
        """
        :param key: The encryption key
        :return: AESCipher instance
        """
        self.__class__._deprecation_notice()
        self.key = key

    def build_cipher(self, alg='aes_128_cbc'):
        """
        :param alg: cipher algorithm
        :return: A Cipher instance
        """
        self.__class__._deprecation_notice()
        typ, bits, cmode = alg.lower().split('_')
        bits = int(bits)
        iv = _os.urandom(self.AES_BLOCK_SIZE)

        if len(iv) != self.AES_BLOCK_SIZE:
            raise Exception('Wrong iv size: {}'.format(len(iv)))

        if bits not in _ciphers.algorithms.AES.key_sizes:
            raise Exception('Unsupported key length: {}'.format(bits))

        if len(self.key) != bits / 8:
            raise Exception('Wrong Key length: {}'.format(len(self.key)))

        try:
            mode = self.POSTFIX_MODE[cmode]
        except KeyError:
            raise Exception('Unsupported chaining mode: {}'.format(cmode))

        cipher = _ciphers.Cipher(
                _ciphers.algorithms.AES(self.key),
                mode(iv),
                backend=_backends.default_backend())

        return cipher, iv

    def encrypt(self, msg, alg='aes_128_cbc', padding='PKCS#7', b64enc=True,
                block_size=AES_BLOCK_SIZE):
        """
        :param key: The encryption key
        :param msg: Message to be encrypted
        :param padding: Which padding that should be used
        :param b64enc: Whether the result should be base64encoded
        :param block_size: If PKCS#7 padding which block size to use
        :return: The encrypted message
        """
        self.__class__._deprecation_notice()
        if padding == 'PKCS#7':
            _block_size = block_size
        elif padding == 'PKCS#5':
            _block_size = 8
        else:
            _block_size = 0

        if _block_size:
            plen = _block_size - (len(msg) % _block_size)
            c = chr(plen).encode()
            msg += c * plen

        cipher, iv = self.build_cipher(alg)
        encryptor = cipher.encryptor()
        cmsg = iv + encryptor.update(msg) + encryptor.finalize()

        if b64enc:
            enc_msg = _base64.b64encode(cmsg)
        else:
            enc_msg = cmsg

        return enc_msg

    def decrypt(self, msg, alg='aes_128_cbc', padding='PKCS#7', b64dec=True):
        """
        :param key: The encryption key
        :param msg: Base64 encoded message to be decrypted
        :return: The decrypted message
        """
        self.__class__._deprecation_notice()
        data = _base64.b64decode(msg) if b64dec else msg

        cipher, iv = self.build_cipher(alg=alg)
        decryptor = cipher.decryptor()
        res = decryptor.update(data)[self.AES_BLOCK_SIZE:]
        res += decryptor.finalize()
        if padding in ['PKCS#5', 'PKCS#7']:
            idx = bytearray(res)[-1]
            res = res[:-idx]
        return res


class Default(Fernet):
    """Default class is saml2.cryptography.symmetric.Fernet"""



import calendar
from six.moves.urllib.parse import urlparse
import re
import struct
import base64
import time
from ipaddress import AddressValueError
from ipaddress import IPv4Address
from ipaddress import IPv6Address

from saml2 import time_util

XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
XSI_NIL = '{%s}nil' % XSI_NAMESPACE
# ---------------------------------------------------------


class NotValid(Exception):
    pass


class OutsideCardinality(Exception):
    pass


class MustValueError(ValueError):
    pass


class ShouldValueError(ValueError):
    pass


class ResponseLifetimeExceed(Exception):
    pass


class ToEarly(Exception):
    pass

# --------------------- validators -------------------------------------
#

NCNAME = re.compile(r"(?P<NCName>[a-zA-Z_](\w|[_.-])*)")


def valid_ncname(name):
    match = NCNAME.match(name)
    #if not match:                      # hack for invalid authnRequest/ID from meteor saml lib
    #    raise NotValid("NCName")
    return True


def valid_id(oid):
    valid_ncname(oid)


def valid_any_uri(item):
    """very simplistic, ..."""
    try:
        part = urlparse(item)
    except Exception:
        raise NotValid("AnyURI")

    if part[0] == "urn" and part[1] == "":  # A urn
        return True
    # elif part[1] == "localhost" or part[1] == "127.0.0.1":
    #     raise NotValid("AnyURI")

    return True


def valid_date_time(item):
    try:
        time_util.str_to_time(item)
    except Exception:
        raise NotValid("dateTime")
    return True


def valid_url(url):
    try:
        _ = urlparse(url)
    except Exception:
        raise NotValid("URL")

    # if part[1] == "localhost" or part[1] == "127.0.0.1":
    #     raise NotValid("URL")
    return True


def validate_on_or_after(not_on_or_after, slack):
    if not_on_or_after:
        now = time_util.utc_now()
        nooa = calendar.timegm(time_util.str_to_time(not_on_or_after))
        if now > nooa + slack:
            now_str=time.strftime('%Y-%m-%dT%H:%M:%SZ', time.gmtime(now))
            raise ResponseLifetimeExceed(
                "Can't use response, too old (now=%s + slack=%d > " \
                "not_on_or_after=%s" % (now_str, slack, not_on_or_after))
        return nooa
    else:
        return False


def validate_before(not_before, slack):
    if not_before:
        now = time_util.utc_now()
        nbefore = calendar.timegm(time_util.str_to_time(not_before))
        if nbefore > now + slack:
            now_str = time.strftime('%Y-%m-%dT%H:%M:%SZ', time.gmtime(now))
            raise ToEarly("Can't use response yet: (now=%s + slack=%d) "
                          "<= notbefore=%s" % (now_str, slack, not_before))
    return True


def valid_address(address):
    """Validate IPv4/IPv6 addresses."""
    if not (valid_ipv4(address) or valid_ipv6(address)):
        raise NotValid("address")
    return True


def valid_ipv4(address):
    """Validate IPv4 addresses."""
    try:
        IPv4Address(address)
    except AddressValueError:
        return False
    return True


def valid_ipv6(address):
    """Validate IPv6 addresses."""
    is_enclosed_in_brackets = address.startswith("[") and address.endswith("]")
    address_raw = address[1:-1] if is_enclosed_in_brackets else address
    try:
        IPv6Address(address_raw)
    except AddressValueError:
        return False
    return True


def valid_boolean(val):
    vall = val.lower()
    if vall in ["true", "false", "0", "1"]:
        return True
    else:
        raise NotValid("boolean")


def valid_duration(val):
    try:
        time_util.parse_duration(val)
    except Exception:
        raise NotValid("duration")
    return True


def valid_string(val):
    """ Expects unicode
    Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] |
                    [#x10000-#x10FFFF]
    """
    for char in val:
        try:
            char = ord(char)
        except TypeError:
            raise NotValid("string")
        if char == 0x09 or char == 0x0A or char == 0x0D:
            continue
        elif 0x20 <= char <= 0xD7FF:
            continue
        elif 0xE000 <= char <= 0xFFFD:
            continue
        elif 0x10000 <= char <= 0x10FFFF:
            continue
        else:
            raise NotValid("string")
    return True


def valid_unsigned_short(val):
    try:
        struct.pack("H", int(val))
    except struct.error:
        raise NotValid("unsigned short")
    except ValueError:
        raise NotValid("unsigned short")

    return True


def valid_positive_integer(val):
    try:
        integer = int(val)
    except ValueError:
        raise NotValid("positive integer")

    if integer > 0:
        return True
    else:
        raise NotValid("positive integer")


def valid_non_negative_integer(val):
    try:
        integer = int(val)
    except ValueError:
        raise NotValid("non negative integer")

    if integer < 0:
        raise NotValid("non negative integer")
    return True


def valid_integer(val):
    try:
        int(val)
    except ValueError:
        raise NotValid("integer")
    return True


def valid_base64(val):
    try:
        base64.b64decode(val)
    except Exception:
        raise NotValid("base64")
    return True


def valid_qname(val):
    """ A qname is either
        NCName or
        NCName ':' NCName
    """

    try:
        (prefix, localpart) = val.split(":")
        return valid_ncname(prefix) and valid_ncname(localpart)
    except ValueError:
        return valid_ncname(val)


def valid_anytype(val):
    """ Goes through all known type validators

    :param val: The value to validate
    :return: True is value is valid otherwise an exception is raised
    """
    for validator in VALIDATOR.values():
        if validator == valid_anytype:  # To hinder recursion
            continue
        try:
            if validator(val):
                return True
        except NotValid:
            pass

    if isinstance(val, type):
        return True

    raise NotValid("AnyType")

# -----------------------------------------------------------------------------

VALIDATOR = {
    "ID": valid_id,
    "NCName": valid_ncname,
    "dateTime": valid_date_time,
    "anyURI": valid_any_uri,
    "nonNegativeInteger": valid_non_negative_integer,
    "PositiveInteger": valid_positive_integer,
    "boolean": valid_boolean,
    "unsignedShort": valid_unsigned_short,
    "duration": valid_duration,
    "base64Binary": valid_base64,
    "integer": valid_integer,
    "QName": valid_qname,
    "anyType": valid_anytype,
    "string": valid_string,
}

# -----------------------------------------------------------------------------


def validate_value_type(value, spec):
    """
    c_value_type = {'base': 'string', 'enumeration': ['Permit', 'Deny',
                                                      'Indeterminate']}
        {'member': 'anyURI', 'base': 'list'}
        {'base': 'anyURI'}
        {'base': 'NCName'}
        {'base': 'string'}
    """
    if "maxlen" in spec:
        return len(value) <= int(spec["maxlen"])

    if spec["base"] == "string":
        if "enumeration" in spec:
            if value not in spec["enumeration"]:
                raise NotValid("value not in enumeration")
        else:
            return valid_string(value)
    elif spec["base"] == "list":  # comma separated list of values
        for val in [v.strip() for v in value.split(",")]:
            valid(spec["member"], val)
    else:
        return valid(spec["base"], value)

    return True


def valid(typ, value):
    try:
        return VALIDATOR[typ](value)
    except KeyError:
        try:
            (_namespace, typ) = typ.split(":")
        except ValueError:
            if typ == "":
                typ = "string"
        return VALIDATOR[typ](value)


def _valid_instance(instance, val):
    try:
        val.verify()
    except NotValid as exc:
        raise NotValid("Class '%s' instance: %s" % (
            instance.__class__.__name__, exc.args[0]))
    except OutsideCardinality as exc:
        raise NotValid(
            "Class '%s' instance cardinality error: %s" % (
                instance.__class__.__name__, exc.args[0]))

ERROR_TEXT = "Wrong type of value '%s' on attribute '%s' expected it to be %s"


def valid_instance(instance):
    instclass = instance.__class__
    class_name = instclass.__name__

    # if instance.text:
    #     _has_val = True
    # else:
    #     _has_val = False

    if instclass.c_value_type and instance.text:
        try:
            validate_value_type(instance.text.strip(),
                                instclass.c_value_type)
        except NotValid as exc:
            raise NotValid("Class '%s' instance: %s" % (class_name,
                                                        exc.args[0]))

    for (name, typ, required) in instclass.c_attributes.values():
        value = getattr(instance, name, '')
        if required and not value:
            txt = "Required value on property '%s' missing" % name
            raise MustValueError("Class '%s' instance: %s" % (class_name, txt))

        if value:
            try:
                if isinstance(typ, type):
                    if typ.c_value_type:
                        spec = typ.c_value_type
                    else:
                        spec = {"base": "string"}  # do I need a default

                    validate_value_type(value, spec)
                else:
                    valid(typ, value)
            except (NotValid, ValueError) as exc:
                txt = ERROR_TEXT % (value, name, exc.args[0])
                raise NotValid("Class '%s' instance: %s" % (class_name, txt))

    for (name, _spec) in instclass.c_children.values():
        value = getattr(instance, name, '')

        try:
            _card = instclass.c_cardinality[name]
            try:
                _cmin = _card["min"]
            except KeyError:
                _cmin = None
            try:
                _cmax = _card["max"]
            except KeyError:
                _cmax = None
        except KeyError:
            _cmin = _cmax = _card = None

        if value:
            #_has_val = True
            if isinstance(value, list):
                _list = True
                vlen = len(value)
            else:
                _list = False
                vlen = 1

            if _card:
                if _cmin is not None and _cmin > vlen:
                    raise NotValid(
                        "Class '%s' instance cardinality error: %s" % (
                            class_name, "less then min (%s<%s)" % (vlen,
                                                                   _cmin)))
                if _cmax is not None and vlen > _cmax:
                    raise NotValid(
                        "Class '%s' instance cardinality error: %s" % (
                            class_name, "more then max (%s>%s)" % (vlen,
                                                                   _cmax)))

            if _list:
                for val in value:
                    # That it is the right class is handled elsewhere
                    _valid_instance(instance, val)
            else:
                _valid_instance(instance, value)
        else:
            if _cmin:
                raise NotValid(
                    "Class '%s' instance cardinality error: %s" % (
                        class_name, "too few values on %s" % name))

    return True


def valid_domain_name(dns_name):
    m = re.match(
        r"^[a-z0-9]+([-.]{ 1 }[a-z0-9]+).[a-z]{2,5}(:[0-9]{1,5})?(\/.)?$",
        dns_name, re.I)
    if not m:
        raise ValueError("Not a proper domain name")



#!/usr/bin/env python

#
# Generated Tue Jul 18 15:03:44 2017 by parse_xsd.py version 0.5.
#

import saml2
from saml2 import SamlBase


NAMESPACE = 'http://eidas.europa.eu/saml-extensions'

class SPTypeType_(SamlBase):
    """The http://eidas.europa.eu/saml-extensions:SPTypeType element """

    c_tag = 'SPTypeType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xsd:string', 'enumeration': ['public', 'private']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def sp_type_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SPTypeType_, xml_string)


class SPType(SPTypeType_):
    """The http://eidas.europa.eu/saml-extensions:SPType element """

    c_tag = 'SPType'
    c_namespace = NAMESPACE
    c_children = SPTypeType_.c_children.copy()
    c_attributes = SPTypeType_.c_attributes.copy()
    c_child_order = SPTypeType_.c_child_order[:]
    c_cardinality = SPTypeType_.c_cardinality.copy()

def sp_type_from_string(xml_string):
    return saml2.create_class_from_xml_string(SPType, xml_string)


ELEMENT_FROM_STRING = {
    SPType.c_tag: sp_type_from_string,
    SPTypeType_.c_tag: sp_type_type__from_string,
}

ELEMENT_BY_TAG = {
    'SPType': SPType,
    'SPTypeType': SPTypeType_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



#!/usr/bin/env python

#
# Generated Mon May  2 14:23:33 2011 by parse_xsd.py version 0.4.
#

import saml2
from saml2 import SamlBase
from saml2 import xmldsig as ds

NAMESPACE = 'http://www.w3.org/2001/04/xmlenc#'

class KeySizeType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:KeySizeType element """

    c_tag = 'KeySizeType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'integer'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def key_size_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeySizeType_, xml_string)


class CipherValue(SamlBase):

    c_tag = 'CipherValue'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'base64Binary'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def cipher_value_from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherValue, xml_string)


class TransformsType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:TransformsType element """

    c_tag = 'TransformsType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2000/09/xmldsig#}Transform'] = ('transform', [ds.Transform])
    c_cardinality['transform'] = {"min":1}
    c_child_order.extend(['transform'])

    def __init__(self,
            transform=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.transform=transform or []

def transforms_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(TransformsType_, xml_string)


class KA_Nonce(SamlBase):

    c_tag = 'KA_Nonce'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'base64Binary'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def k_a__nonce_from_string(xml_string):
    return saml2.create_class_from_xml_string(KA_Nonce, xml_string)


class OriginatorKeyInfo(ds.KeyInfo):

    c_tag = 'OriginatorKeyInfo'
    c_namespace = NAMESPACE
    c_children = ds.KeyInfo.c_children.copy()
    c_attributes = ds.KeyInfo.c_attributes.copy()
    c_child_order = ds.KeyInfo.c_child_order[:]
    c_cardinality = ds.KeyInfo.c_cardinality.copy()

def originator_key_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(OriginatorKeyInfo, xml_string)


class RecipientKeyInfo(ds.KeyInfo):

    c_tag = 'RecipientKeyInfo'
    c_namespace = NAMESPACE
    c_children = ds.KeyInfo.c_children.copy()
    c_attributes = ds.KeyInfo.c_attributes.copy()
    c_child_order = ds.KeyInfo.c_child_order[:]
    c_cardinality = ds.KeyInfo.c_cardinality.copy()

def recipient_key_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(RecipientKeyInfo, xml_string)


class AgreementMethodType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:AgreementMethodType element """

    c_tag = 'AgreementMethodType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}KA_Nonce'] = ('k_a__nonce', KA_Nonce)
    c_cardinality['k_a__nonce'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2001/04/xmlenc#}OriginatorKeyInfo'] = ('originator_key_info', OriginatorKeyInfo)
    c_cardinality['originator_key_info'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2001/04/xmlenc#}RecipientKeyInfo'] = ('recipient_key_info', RecipientKeyInfo)
    c_cardinality['recipient_key_info'] = {"min":0, "max":1}
    c_attributes['Algorithm'] = ('algorithm', 'anyURI', True)
    c_child_order.extend(['k_a__nonce', 'originator_key_info', 'recipient_key_info'])

    def __init__(self,
            k_a__nonce=None,
            originator_key_info=None,
            recipient_key_info=None,
            algorithm=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.k_a__nonce=k_a__nonce
        self.originator_key_info=originator_key_info
        self.recipient_key_info=recipient_key_info
        self.algorithm=algorithm

def agreement_method_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AgreementMethodType_, xml_string)


class ReferenceType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:ReferenceType element """

    c_tag = 'ReferenceType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['URI'] = ('uri', 'anyURI', True)

    def __init__(self,
            uri=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.uri=uri

def reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ReferenceType_, xml_string)


class EncryptionPropertyType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionPropertyType element """

    c_tag = 'EncryptionPropertyType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Target'] = ('target', 'anyURI', False)
    c_attributes['Id'] = ('id', 'ID', False)

    def __init__(self,
            target=None,
            id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.target=target
        self.id=id

def encryption_property_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionPropertyType_, xml_string)


class KeySize(KeySizeType_):

    c_tag = 'KeySize'
    c_namespace = NAMESPACE
    c_children = KeySizeType_.c_children.copy()
    c_attributes = KeySizeType_.c_attributes.copy()
    c_child_order = KeySizeType_.c_child_order[:]
    c_cardinality = KeySizeType_.c_cardinality.copy()

def key_size_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeySize, xml_string)


class OAEPparams(SamlBase):

    c_tag = 'OAEPparams'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'base64Binary'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def oae_pparams_from_string(xml_string):
    return saml2.create_class_from_xml_string(OAEPparams, xml_string)


class EncryptionMethodType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionMethodType element """

    c_tag = 'EncryptionMethodType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}KeySize'] = ('key_size', KeySize)
    c_cardinality['key_size'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2001/04/xmlenc#}OAEPparams'] = ('oae_pparams', OAEPparams)
    c_cardinality['oae_pparams'] = {"min":0, "max":1}
    c_attributes['Algorithm'] = ('algorithm', 'anyURI', True)
    c_child_order.extend(['key_size', 'oae_pparams'])

    def __init__(self,
            key_size=None,
            oae_pparams=None,
            algorithm=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.key_size=key_size
        self.oae_pparams=oae_pparams
        self.algorithm=algorithm

def encryption_method_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionMethodType_, xml_string)


class Transforms(TransformsType_):

    c_tag = 'Transforms'
    c_namespace = NAMESPACE
    c_children = TransformsType_.c_children.copy()
    c_attributes = TransformsType_.c_attributes.copy()
    c_child_order = TransformsType_.c_child_order[:]
    c_cardinality = TransformsType_.c_cardinality.copy()

def transforms_from_string(xml_string):
    return saml2.create_class_from_xml_string(Transforms, xml_string)


class CipherReferenceType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:CipherReferenceType element """

    c_tag = 'CipherReferenceType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}Transforms'] = ('transforms', Transforms)
    c_cardinality['transforms'] = {"min":0, "max":1}
    c_attributes['URI'] = ('uri', 'anyURI', True)
    c_child_order.extend(['transforms'])

    def __init__(self,
            transforms=None,
            uri=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.transforms=transforms
        self.uri=uri

def cipher_reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherReferenceType_, xml_string)


class EncryptionMethod(EncryptionMethodType_):

    c_tag = 'EncryptionMethod'
    c_namespace = NAMESPACE
    c_children = EncryptionMethodType_.c_children.copy()
    c_attributes = EncryptionMethodType_.c_attributes.copy()
    c_child_order = EncryptionMethodType_.c_child_order[:]
    c_cardinality = EncryptionMethodType_.c_cardinality.copy()

def encryption_method_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionMethod, xml_string)



class AgreementMethod(AgreementMethodType_):
    """The http://www.w3.org/2001/04/xmlenc#:AgreementMethod element """

    c_tag = 'AgreementMethod'
    c_namespace = NAMESPACE
    c_children = AgreementMethodType_.c_children.copy()
    c_attributes = AgreementMethodType_.c_attributes.copy()
    c_child_order = AgreementMethodType_.c_child_order[:]
    c_cardinality = AgreementMethodType_.c_cardinality.copy()

def agreement_method_from_string(xml_string):
    return saml2.create_class_from_xml_string(AgreementMethod, xml_string)


class DataReference(ReferenceType_):

    c_tag = 'DataReference'
    c_namespace = NAMESPACE
    c_children = ReferenceType_.c_children.copy()
    c_attributes = ReferenceType_.c_attributes.copy()
    c_child_order = ReferenceType_.c_child_order[:]
    c_cardinality = ReferenceType_.c_cardinality.copy()

def data_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(DataReference, xml_string)


class KeyReference(ReferenceType_):

    c_tag = 'KeyReference'
    c_namespace = NAMESPACE
    c_children = ReferenceType_.c_children.copy()
    c_attributes = ReferenceType_.c_attributes.copy()
    c_child_order = ReferenceType_.c_child_order[:]
    c_cardinality = ReferenceType_.c_cardinality.copy()

def key_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyReference, xml_string)


class ReferenceList(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:ReferenceList element """

    c_tag = 'ReferenceList'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}DataReference'] = ('data_reference', [DataReference])
    c_cardinality['data_reference'] = {"min":0}
    c_children['{http://www.w3.org/2001/04/xmlenc#}KeyReference'] = ('key_reference', [KeyReference])
    c_cardinality['key_reference'] = {"min":0}
    c_child_order.extend(['data_reference', 'key_reference'])

    def __init__(self,
            data_reference=None,
            key_reference=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.data_reference=data_reference or []
        self.key_reference=key_reference or []

def reference_list_from_string(xml_string):
    return saml2.create_class_from_xml_string(ReferenceList, xml_string)


class EncryptionProperty(EncryptionPropertyType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionProperty element """

    c_tag = 'EncryptionProperty'
    c_namespace = NAMESPACE
    c_children = EncryptionPropertyType_.c_children.copy()
    c_attributes = EncryptionPropertyType_.c_attributes.copy()
    c_child_order = EncryptionPropertyType_.c_child_order[:]
    c_cardinality = EncryptionPropertyType_.c_cardinality.copy()

def encryption_property_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionProperty, xml_string)


class CipherReference(CipherReferenceType_):
    """The http://www.w3.org/2001/04/xmlenc#:CipherReference element """

    c_tag = 'CipherReference'
    c_namespace = NAMESPACE
    c_children = CipherReferenceType_.c_children.copy()
    c_attributes = CipherReferenceType_.c_attributes.copy()
    c_child_order = CipherReferenceType_.c_child_order[:]
    c_cardinality = CipherReferenceType_.c_cardinality.copy()

def cipher_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherReference, xml_string)


class EncryptionPropertiesType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionPropertiesType element """

    c_tag = 'EncryptionPropertiesType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}EncryptionProperty'] = ('encryption_property', [EncryptionProperty])
    c_cardinality['encryption_property'] = {"min":1}
    c_attributes['Id'] = ('id', 'ID', False)
    c_child_order.extend(['encryption_property'])

    def __init__(self,
            encryption_property=None,
            id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.encryption_property=encryption_property or []
        self.id=id

def encryption_properties_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionPropertiesType_, xml_string)


class CipherDataType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:CipherDataType element """

    c_tag = 'CipherDataType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}CipherValue'] = ('cipher_value', CipherValue)
    c_cardinality['cipher_value'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2001/04/xmlenc#}CipherReference'] = ('cipher_reference', CipherReference)
    c_cardinality['cipher_reference'] = {"min":0, "max":1}
    c_child_order.extend(['cipher_value', 'cipher_reference'])

    def __init__(self,
            cipher_value=None,
            cipher_reference=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.cipher_value=cipher_value
        self.cipher_reference=cipher_reference

def cipher_data_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherDataType_, xml_string)


class EncryptionProperties(EncryptionPropertiesType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptionProperties element """

    c_tag = 'EncryptionProperties'
    c_namespace = NAMESPACE
    c_children = EncryptionPropertiesType_.c_children.copy()
    c_attributes = EncryptionPropertiesType_.c_attributes.copy()
    c_child_order = EncryptionPropertiesType_.c_child_order[:]
    c_cardinality = EncryptionPropertiesType_.c_cardinality.copy()

def encryption_properties_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionProperties, xml_string)


class CipherData(CipherDataType_):
    """The http://www.w3.org/2001/04/xmlenc#:CipherData element """

    c_tag = 'CipherData'
    c_namespace = NAMESPACE
    c_children = CipherDataType_.c_children.copy()
    c_attributes = CipherDataType_.c_attributes.copy()
    c_child_order = CipherDataType_.c_child_order[:]
    c_cardinality = CipherDataType_.c_cardinality.copy()

def cipher_data_from_string(xml_string):
    return saml2.create_class_from_xml_string(CipherData, xml_string)


class EncryptedType_(SamlBase):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedType element """

    c_tag = 'EncryptedType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}EncryptionMethod'] = ('encryption_method', EncryptionMethod)
    c_cardinality['encryption_method'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2000/09/xmldsig#}KeyInfo'] = ('key_info', ds.KeyInfo)
    c_cardinality['key_info'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2001/04/xmlenc#}CipherData'] = ('cipher_data', CipherData)
    c_children['{http://www.w3.org/2001/04/xmlenc#}EncryptionProperties'] = ('encryption_properties', EncryptionProperties)
    c_cardinality['encryption_properties'] = {"min":0, "max":1}
    c_attributes['Id'] = ('id', 'ID', False)
    c_attributes['Type'] = ('type', 'anyURI', False)
    c_attributes['MimeType'] = ('mime_type', 'string', False)
    c_attributes['Encoding'] = ('encoding', 'anyURI', False)
    c_child_order.extend(['encryption_method', 'key_info', 'cipher_data', 'encryption_properties'])

    def __init__(self,
            encryption_method=None,
            key_info=None,
            cipher_data=None,
            encryption_properties=None,
            id=None,
            type=None,
            mime_type=None,
            encoding=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.encryption_method=encryption_method
        self.key_info=key_info
        self.cipher_data=cipher_data
        self.encryption_properties=encryption_properties
        self.id=id
        self.type=type
        self.mime_type=mime_type
        self.encoding=encoding


class EncryptedDataType_(EncryptedType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedDataType element """

    c_tag = 'EncryptedDataType'
    c_namespace = NAMESPACE
    c_children = EncryptedType_.c_children.copy()
    c_attributes = EncryptedType_.c_attributes.copy()
    c_child_order = EncryptedType_.c_child_order[:]
    c_cardinality = EncryptedType_.c_cardinality.copy()

def encrypted_data_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedDataType_, xml_string)


class CarriedKeyName(SamlBase):

    c_tag = 'CarriedKeyName'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def carried_key_name_from_string(xml_string):
    return saml2.create_class_from_xml_string(CarriedKeyName, xml_string)


class EncryptedKeyType_(EncryptedType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedKeyType element """

    c_tag = 'EncryptedKeyType'
    c_namespace = NAMESPACE
    c_children = EncryptedType_.c_children.copy()
    c_attributes = EncryptedType_.c_attributes.copy()
    c_child_order = EncryptedType_.c_child_order[:]
    c_cardinality = EncryptedType_.c_cardinality.copy()
    c_children['{http://www.w3.org/2001/04/xmlenc#}ReferenceList'] = ('reference_list', ReferenceList)
    c_cardinality['reference_list'] = {"min":0, "max":1}
    c_children['{http://www.w3.org/2001/04/xmlenc#}CarriedKeyName'] = ('carried_key_name', CarriedKeyName)
    c_cardinality['carried_key_name'] = {"min":0, "max":1}
    c_attributes['Recipient'] = ('recipient', 'string', False)
    c_child_order.extend(['reference_list', 'carried_key_name'])

    def __init__(self,
            reference_list=None,
            carried_key_name=None,
            recipient=None,
            encryption_method=None,
            key_info=None,
            cipher_data=None,
            encryption_properties=None,
            id=None,
            type=None,
            mime_type=None,
            encoding=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        EncryptedType_.__init__(self,
                encryption_method=encryption_method,
                key_info=key_info,
                cipher_data=cipher_data,
                encryption_properties=encryption_properties,
                id=id,
                type=type,
                mime_type=mime_type,
                encoding=encoding,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.reference_list=reference_list
        self.carried_key_name=carried_key_name
        self.recipient=recipient

def encrypted_key_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedKeyType_, xml_string)


class EncryptedData(EncryptedDataType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedData element """

    c_tag = 'EncryptedData'
    c_namespace = NAMESPACE
    c_children = EncryptedDataType_.c_children.copy()
    c_attributes = EncryptedDataType_.c_attributes.copy()
    c_child_order = EncryptedDataType_.c_child_order[:]
    c_cardinality = EncryptedDataType_.c_cardinality.copy()

def encrypted_data_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedData, xml_string)


class EncryptedKey(EncryptedKeyType_):
    """The http://www.w3.org/2001/04/xmlenc#:EncryptedKey element """

    c_tag = 'EncryptedKey'
    c_namespace = NAMESPACE
    c_children = EncryptedKeyType_.c_children.copy()
    c_attributes = EncryptedKeyType_.c_attributes.copy()
    c_child_order = EncryptedKeyType_.c_child_order[:]
    c_cardinality = EncryptedKeyType_.c_cardinality.copy()

def encrypted_key_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptedKey, xml_string)

ds.KeyInfo.c_children['{http://www.w3.org/2000/09/xmlenc#}EncryptedKey'] =  (
    'encrypted_key',
    EncryptedKey)


ELEMENT_FROM_STRING = {
    EncryptionMethodType_.c_tag: encryption_method_type__from_string,
    KeySizeType_.c_tag: key_size_type__from_string,
    CipherData.c_tag: cipher_data_from_string,
    CipherDataType_.c_tag: cipher_data_type__from_string,
    CipherReference.c_tag: cipher_reference_from_string,
    CipherReferenceType_.c_tag: cipher_reference_type__from_string,
    TransformsType_.c_tag: transforms_type__from_string,
    EncryptedData.c_tag: encrypted_data_from_string,
    EncryptedDataType_.c_tag: encrypted_data_type__from_string,
    EncryptedKey.c_tag: encrypted_key_from_string,
    EncryptedKeyType_.c_tag: encrypted_key_type__from_string,
    AgreementMethod.c_tag: agreement_method_from_string,
    AgreementMethodType_.c_tag: agreement_method_type__from_string,
    ReferenceList.c_tag: reference_list_from_string,
    ReferenceType_.c_tag: reference_type__from_string,
    EncryptionProperties.c_tag: encryption_properties_from_string,
    EncryptionPropertiesType_.c_tag: encryption_properties_type__from_string,
    EncryptionProperty.c_tag: encryption_property_from_string,
    EncryptionPropertyType_.c_tag: encryption_property_type__from_string,
    CipherValue.c_tag: cipher_value_from_string,
    KA_Nonce.c_tag: k_a__nonce_from_string,
    OriginatorKeyInfo.c_tag: originator_key_info_from_string,
    RecipientKeyInfo.c_tag: recipient_key_info_from_string,
    KeySize.c_tag: key_size_from_string,
    OAEPparams.c_tag: oae_pparams_from_string,
    Transforms.c_tag: transforms_from_string,
    EncryptionMethod.c_tag: encryption_method_from_string,
    DataReference.c_tag: data_reference_from_string,
    KeyReference.c_tag: key_reference_from_string,
    CarriedKeyName.c_tag: carried_key_name_from_string,
}

ELEMENT_BY_TAG = {
    'EncryptionMethodType': EncryptionMethodType_,
    'KeySizeType': KeySizeType_,
    'CipherData': CipherData,
    'CipherDataType': CipherDataType_,
    'CipherReference': CipherReference,
    'CipherReferenceType': CipherReferenceType_,
    'TransformsType': TransformsType_,
    'EncryptedData': EncryptedData,
    'EncryptedDataType': EncryptedDataType_,
    'EncryptedKey': EncryptedKey,
    'EncryptedKeyType': EncryptedKeyType_,
    'AgreementMethod': AgreementMethod,
    'AgreementMethodType': AgreementMethodType_,
    'ReferenceList': ReferenceList,
    'ReferenceType': ReferenceType_,
    'EncryptionProperties': EncryptionProperties,
    'EncryptionPropertiesType': EncryptionPropertiesType_,
    'EncryptionProperty': EncryptionProperty,
    'EncryptionPropertyType': EncryptionPropertyType_,
    'CipherValue': CipherValue,
    'KA_Nonce': KA_Nonce,
    'OriginatorKeyInfo': OriginatorKeyInfo,
    'RecipientKeyInfo': RecipientKeyInfo,
    'KeySize': KeySize,
    'OAEPparams': OAEPparams,
    'Transforms': Transforms,
    'EncryptionMethod': EncryptionMethod,
    'DataReference': DataReference,
    'KeyReference': KeyReference,
    'CarriedKeyName': CarriedKeyName,
    'EncryptedType': EncryptedType_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)




# -*- encoding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 6
_modified_time = 1357242050.211483
_template_filename=u'templates/root.mako'
_template_uri=u'root.mako'
_template_cache=cache.Cache(__name__, _modified_time)
_source_encoding='utf-8'
_exports = ['css_link', 'pre', 'post', 'css']


def render_body(context,**pageargs):
    context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        def pre():
            return render_pre(context.locals_(__M_locals))
        self = context.get('self', UNDEFINED)
        set = context.get('set', UNDEFINED)
        def post():
            return render_post(context.locals_(__M_locals))
        next = context.get('next', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 1
        self.seen_css = set() 
        
        __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n')
        # SOURCE LINE 10
        __M_writer(u'\n')
        # SOURCE LINE 15
        __M_writer(u'\n')
        # SOURCE LINE 22
        __M_writer(u'\n')
        # SOURCE LINE 25
        __M_writer(u'<html>\n<head><title>IDP test login</title>\n    ')
        # SOURCE LINE 27
        __M_writer(unicode(self.css()))
        __M_writer(u'\n    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />\n</head>\n<body>\n    ')
        # SOURCE LINE 31
        __M_writer(unicode(pre()))
        __M_writer(u'\n')
        # SOURCE LINE 34
        __M_writer(unicode(next.body()))
        __M_writer(u'\n')
        # SOURCE LINE 35
        __M_writer(unicode(post()))
        __M_writer(u'\n</body>\n</html>\n')
        return ''
    finally:
        context.caller_stack._pop_frame()


def render_css_link(context,path,media=''):
    context.caller_stack._push_frame()
    try:
        context._push_buffer()
        self = context.get('self', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n')
        # SOURCE LINE 3
        if path not in self.seen_css:
            # SOURCE LINE 4
            __M_writer(u'        <link rel="stylesheet" type="text/css" href="')
            __M_writer(filters.html_escape(unicode(path)))
            __M_writer(u'" media="')
            __M_writer(unicode(media))
            __M_writer(u'">\n')
            pass
        # SOURCE LINE 6
        __M_writer(u'    ')
        self.seen_css.add(path) 
        
        __M_writer(u'\n')
    finally:
        __M_buf, __M_writer = context._pop_buffer_and_writer()
        context.caller_stack._pop_frame()
    __M_writer(filters.trim(__M_buf.getvalue()))
    return ''


def render_pre(context):
    context.caller_stack._push_frame()
    try:
        context._push_buffer()
        __M_writer = context.writer()
        # SOURCE LINE 11
        __M_writer(u'\n    <div class="header">\n        <h1><a href="/">Login</a></h1>\n    </div>\n')
    finally:
        __M_buf, __M_writer = context._pop_buffer_and_writer()
        context.caller_stack._pop_frame()
    __M_writer(filters.trim(__M_buf.getvalue()))
    return ''


def render_post(context):
    context.caller_stack._push_frame()
    try:
        context._push_buffer()
        __M_writer = context.writer()
        # SOURCE LINE 16
        __M_writer(u'\n    <div>\n        <div class="footer">\n            <p>&#169; Copyright 2011 Ume&#229; Universitet &nbsp;</p>\n        </div>\n    </div>\n')
    finally:
        __M_buf, __M_writer = context._pop_buffer_and_writer()
        context.caller_stack._pop_frame()
    __M_writer(filters.trim(__M_buf.getvalue()))
    return ''


def render_css(context):
    context.caller_stack._push_frame()
    try:
        context._push_buffer()
        def css_link(path,media=''):
            return render_css_link(context,path,media)
        __M_writer = context.writer()
        # SOURCE LINE 8
        __M_writer(u'\n    ')
        # SOURCE LINE 9
        __M_writer(unicode(css_link('/css/main.css', 'screen')))
        __M_writer(u'\n')
    finally:
        __M_buf, __M_writer = context._pop_buffer_and_writer()
        context.caller_stack._pop_frame()
    __M_writer(filters.trim(__M_buf.getvalue()))
    return ''





#!/usr/bin/env python
# This Python file uses the following encoding: utf-8
# ISO 3166-1 country names and codes from http://opencountrycodes.appspot.com/python

COUNTRIES = (
    ("AF", "Afghanistan"),("AX", "Aland Islands"),("AL", "Albania"),
    ("DZ", "Algeria"),("AS", "American Samoa"),("AD", "Andorra"),
    ("AO", "Angola"),("AI", "Anguilla"),("AQ", "Antarctica"),
    ("AG", "Antigua and Barbuda"),("AR", "Argentina"),("AM", "Armenia"),
    ("AW", "Aruba"),("AU", "Australia"),("AT", "Austria"),
    ("AZ", "Azerbaijan"),("BS", "Bahamas"),("BH", "Bahrain"),
    ("BD", "Bangladesh"),("BB", "Barbados"),("BY", "Belarus"),("BE", "Belgium"),
    ("BZ", "Belize"),("BJ", "Benin"),("BM", "Bermuda"),("BT", "Bhutan"),
    ("BO", "Bolivia, Plurinational State of"),
    ("BQ", "Bonaire, Sint Eustatius and Saba"),("BA", "Bosnia and Herzegovina"),
    ("BW", "Botswana"),("BV", "Bouvet Island"),("BR", "Brazil"),
    ("IO", "British Indian Ocean Territory"),("BN", "Brunei Darussalam"),
    ("BG", "Bulgaria"),("BF", "Burkina Faso"),("BI", "Burundi"),
    ("KH", "Cambodia"),("CM", "Cameroon"),("CA", "Canada"),("CV", "Cape Verde"),
    ("KY", "Cayman Islands"),("CF", "Central African Republic"),("TD", "Chad"),
    ("CL", "Chile"),("CN", "China"),("CX", "Christmas Island"),
    ("CC", "Cocos (Keeling) Islands"),("CO", "Colombia"),("KM", "Comoros"),
    ("CG", "Congo"),("CD", "Congo, The Democratic Republic of the"),
    ("CK", "Cook Islands"),("CR", "Costa Rica"),("CI", "Cote D'ivoire"),
    ("HR", "Croatia"),("CU", "Cuba"),("CW", "Curacao"),("CY", "Cyprus"),
    ("CZ", "Czech Republic"),("DK", "Denmark"),("DJ", "Djibouti"),
    ("DM", "Dominica"),("DO", "Dominican Republic"),("EC", "Ecuador"),
    ("EG", "Egypt"),("SV", "El Salvador"),("GQ", "Equatorial Guinea"),
    ("ER", "Eritrea"),("EE", "Estonia"),("ET", "Ethiopia"),
    ("FK", "Falkland Islands (Malvinas)"),("FO", "Faroe Islands"),
    ("FJ", "Fiji"),("FI", "Finland"),("FR", "France"),("GF", "French Guiana"),
    ("PF", "French Polynesia"),("TF", "French Southern Territories"),
    ("GA", "Gabon"),("GM", "Gambia"),("GE", "Georgia"),("DE", "Germany"),
    ("GH", "Ghana"),("GI", "Gibraltar"),("GR", "Greece"),("GL", "Greenland"),
    ("GD", "Grenada"),("GP", "Guadeloupe"),("GU", "Guam"),("GT", "Guatemala"),
    ("GG", "Guernsey"),("GN", "Guinea"),("GW", "Guinea-Bissau"),("GY", "Guyana"),
    ("HT", "Haiti"),("HM", "Heard Island and McDonald Islands"),
    ("VA", "Holy See (Vatican City State)"),("HN", "Honduras"),
    ("HK", "Hong Kong"),("HU", "Hungary"),("IS", "Iceland"),("IN", "India"),
    ("ID", "Indonesia"),("IR", "Iran, Islamic Republic of"),("IQ", "Iraq"),
    ("IE", "Ireland"),("IM", "Isle of Man"),("IL", "Israel"),("IT", "Italy"),
    ("JM", "Jamaica"),("JP", "Japan"),("JE", "Jersey"),("JO", "Jordan"),
    ("KZ", "Kazakhstan"),("KE", "Kenya"),("KI", "Kiribati"),
    ("KP", "Korea, Democratic People's Republic of"),
    ("KR", "Korea, Republic of"),("KW", "Kuwait"),("KG", "Kyrgyzstan"),
    ("LA", "Lao People's Democratic Republic"),("LV", "Latvia"),
    ("LB", "Lebanon"),("LS", "Lesotho"),("LR", "Liberia"),
    ("LY", "Libyan Arab Jamahiriya"),("LI", "Liechtenstein"),
    ("LT", "Lithuania"),("LU", "Luxembourg"),("MO", "Macao"),
    ("MK", "Macedonia, The Former Yugoslav Republic of"),("MG", "Madagascar"),
    ("MW", "Malawi"),("MY", "Malaysia"),("MV", "Maldives"),("ML", "Mali"),
    ("MT", "Malta"),("MH", "Marshall Islands"),("MQ", "Martinique"),
    ("MR", "Mauritania"),("MU", "Mauritius"),("YT", "Mayotte"),("MX", "Mexico"),
    ("FM", "Micronesia, Federated States of"),("MD", "Moldova, Republic of"),
    ("MC", "Monaco"),("MN", "Mongolia"),("ME", "Montenegro"),
    ("MS", "Montserrat"),("MA", "Morocco"),("MZ", "Mozambique"),
    ("MM", "Myanmar"),("NA", "Namibia"),("NR", "Nauru"),("NP", "Nepal"),
    ("NL", "Netherlands"),("NC", "New Caledonia"),("NZ", "New Zealand"),
    ("NI", "Nicaragua"),("NE", "Niger"),("NG", "Nigeria"),("NU", "Niue"),
    ("NF", "Norfolk Island"),("MP", "Northern Mariana Islands"),
    ("NO", "Norway"),("OM", "Oman"),("PK", "Pakistan"),("PW", "Palau"),
    ("PS", "Palestinian Territory, Occupied"),("PA", "Panama"),
    ("PG", "Papua New Guinea"),("PY", "Paraguay"),("PE", "Peru"),
    ("PH", "Philippines"),("PN", "Pitcairn"),("PL", "Poland"),
    ("PT", "Portugal"),("PR", "Puerto Rico"),("QA", "Qatar"),("RE", "Reunion"),
    ("RO", "Romania"),("RU", "Russian Federation"),("RW", "Rwanda"),
    ("BL", "Saint Barthelemy"),
    ("SH", "Saint Helena, Ascension and Tristan Da Cunha"),
    ("KN", "Saint Kitts and Nevis"),("LC", "Saint Lucia"),
    ("MF", "Saint Martin (French Part)"),("PM", "Saint Pierre and Miquelon"),
    ("VC", "Saint Vincent and the Grenadines"),("WS", "Samoa"),
    ("SM", "San Marino"),("ST", "Sao Tome and Principe"),("SA", "Saudi Arabia"),
    ("SN", "Senegal"),("RS", "Serbia"),("SC", "Seychelles"),
    ("SL", "Sierra Leone"),("SG", "Singapore"),
    ("SX", "Sint Maarten (Dutch Part)"),("SK", "Slovakia"),("SI", "Slovenia"),
    ("SB", "Solomon Islands"),("SO", "Somalia"),("ZA", "South Africa"),
    ("GS", "South Georgia and the South Sandwich Islands"),("ES", "Spain"),
    ("LK", "Sri Lanka"),("SD", "Sudan"),("SR", "Suriname"),
    ("SJ", "Svalbard and Jan Mayen"),("SZ", "Swaziland"),("SE", "Sweden"),
    ("CH", "Switzerland"),("SY", "Syrian Arab Republic"),
    ("TW", "Taiwan, Province of China"),("TJ", "Tajikistan"),
    ("TZ", "Tanzania, United Republic of"),("TH", "Thailand"),
    ("TL", "Timor-Leste"),("TG", "Togo"),("TK", "Tokelau"),("TO", "Tonga"),
    ("TT", "Trinidad and Tobago"),("TN", "Tunisia"),("TR", "Turkey"),
    ("TM", "Turkmenistan"),("TC", "Turks and Caicos Islands"),("TV", "Tuvalu"),
    ("UG", "Uganda"),("UA", "Ukraine"),("AE", "United Arab Emirates"),
    ("GB", "United Kingdom"),("US", "United States"),
    ("UM", "United States Minor Outlying Islands"),("UY", "Uruguay"),
    ("UZ", "Uzbekistan"),("VU", "Vanuatu"),
    ("VE", "Venezuela, Bolivarian Republic of"),("VN", "Viet Nam"),
    ("VG", "Virgin Islands, British"),("VI", "Virgin Islands, U.S."),
    ("WF", "Wallis and Futuna"),("EH", "Western Sahara"),("YE", "Yemen"),
    ("ZM", "Zambia"),("ZW", "Zimbabwe"),)

D_COUNTRIES = dict(COUNTRIES)